#!/bin/sh
### BEGIN INIT INFO
# Provides:          shellinabox
# Required-Start:    $network $remote_fs
# Required-Stop:     $network $remote_fs
# Default-Start:     2 3 4 5
# Default-Stop:      0 1 6
# Short-Description: Shell In A Box Daemon
# Description:       Daemon for publishing a login shell at http://localhost:SHELLINABOX_PORT
### END INIT INFO

DESC="Shell In A Box Daemon"
NAME="shellinabox"
DAEMON="/usr/bin/shellinaboxd"
PIDFILE="/var/run/shellinaboxd.pid"
SCRIPTNAME=/etc/init.d/$NAME

[ -x $DAEMON ] || exit 0

SHELLINABOX_PORT="4200"
SHELLINABOX_CONFIG_DIR="/etc/shellinabox"

d_ssl() {
	if [ ! -d $SHELLINABOX_CONFIG_DIR ] ; then
		mkdir -p $SHELLINABOX_CONFIG_DIR
	fi
	if [ ! -f $SHELLINABOX_CONFIG_DIR/certificate.pem ] ; then
		echo "Generating self signed certificate for shellinabox"
		cd $SHELLINABOX_CONFIG_DIR
		openssl req -new -newkey rsa:1024 -nodes -x509 -subj "/C=NO/ST=Denial/L=NO/O=NO/CN=vuplus.box" -keyout server.key  -out server.crt 2>&1 >/dev/null 2>&1 >/dev/null
		cat server.key server.crt > certificate.pem
		rm server.key server.crt
	fi
}

d_start() {
	d_ssl
	#HACK ALERT
	if pidof -x "opkg" >/dev/null ; then
		exit 0
	fi
	start-stop-daemon --start --pidfile $PIDFILE --exec $DAEMON -o -- --cert $SHELLINABOX_CONFIG_DIR -q --background=$PIDFILE -p $SHELLINABOX_PORT --user-css Normal:+$SHELLINABOX_CONFIG_DIR/vti.css
}

d_stop() {
	start-stop-daemon --stop --oknodo --pidfile "$PIDFILE"
	rm -f "$PIDFILE"
}

d_status() {
	[ -r "$PIDFILE" ] && kill -0 `cat "$PIDFILE"` &&
	echo "$DESC is running" || echo "$DESC is not running"
}

case "$1" in
	start)
		echo "Starting $DESC $NAME" ;
		d_start ;
		;;
	stop)
		echo "Stopping $DESC $NAME" ;
		d_stop ;
		;;
	restart|force-reload)
		echo "Restarting $DESC $NAME" ;
		d_stop ;
		d_start ;
		;;
	status)
		d_status ;
		;;
	*)
		echo "Usage: $SCRIPTNAME {start|stop|restart|}" >&2
		exit 1
		;;
esac

exit 0
