# -*- coding: utf-8 -*-
from Components.Language import language
from Tools.Directories import resolveFilename, SCOPE_PLUGINS, SCOPE_LANGUAGE
from os import environ as os_environ
import gettext

# Config
from Components.config import config, ConfigSet, ConfigSubsection, ConfigText, ConfigNumber, ConfigYesNo, ConfigSelection, ConfigSelectionNumber

def localeInit():
	lang = language.getLanguage()[:2] # getLanguage returns e.g. "fi_FI" for "language_country"
	os_environ["LANGUAGE"] = lang # Enigma doesn't set this (or LC_ALL, LC_MESSAGES, LANG). gettext needs it!
	gettext.bindtextdomain("EPGSearch", resolveFilename(SCOPE_PLUGINS, "Extensions/EPGSearch/locale"))

def _(txt):
	t = gettext.dgettext("EPGSearch", txt)
	if t == txt:
		t = gettext.gettext(txt)
	return t

localeInit()
language.addCallback(localeInit)

config.plugins.epgsearch = ConfigSubsection()
config.plugins.epgsearch.history = ConfigSet(choices = [])
config.plugins.epgsearch.history_length = ConfigNumber(default = 10)
config.plugins.epgsearch.add_search_to_epg = ConfigYesNo(default = True)
config.plugins.epgsearch.search_type = ConfigSelection(default = "ask", choices = [
	("ask", _("Ask before search")),
	("title", _("Title")),
	("shortdesc", _("Short description")),
	("title_shortdesc", _("Title and short description")),
	("extdesc", _("Extended description")),
	("full", _("Title, short and extended description"))])
config.plugins.epgsearch.search_casetype = ConfigYesNo(default = False)
config.plugins.epgsearch.limit_to_bouquet = ConfigYesNo(default = False)
config.plugins.epgsearch.save_any_search_string = ConfigYesNo(default = False)
config.plugins.epgsearch.show_short_description = ConfigYesNo(default = True)
config.plugins.epgsearch.show_popup_time = ConfigSelectionNumber(default = 4, min = 0 , max = 60, stepwidth = 2, wraparound = True)
MATCH_ANY = "any"
MATCH_BEGIN = "begin"
MATCH_EXACT = "exact"
config.plugins.epgsearch.match_type = ConfigSelection(default = MATCH_ANY, choices = [(MATCH_ANY, _("any substring")), (MATCH_BEGIN, _("beginning of a word")), (MATCH_EXACT, _("exact match"))])
