#!/usr/bin/env python
# -*- coding: utf-8 -*-

from skin import parseColor, lookupScreen
from Components.config import config, getConfigListEntry, ConfigSubsection, ConfigSelection, ConfigClock, ConfigYesNo, ConfigText, ConfigDirectory
from Components.ConfigList import ConfigList, ConfigListScreen
from Components.Pixmap import Pixmap
from Components.Button import Button
from Components.Label import Label
from Components.ActionMap import ActionMap, HelpableActionMap
from Components.HTMLComponent import HTMLComponent
from Components.GUIComponent import GUIComponent
from Components.EpgList import Rect
from Components.Sources.Event import Event
from Components.Sources.ServiceEvent import ServiceEvent
from Components.Sources.StaticText import StaticText
from Components.MultiContent import MultiContentEntryText, MultiContentEntryPixmapAlphaTest, MultiContentEntryPixmapAlphaBlend
from Components.TimerList import TimerList
from Components.TimerSanityCheck import TimerSanityCheck
from Components.UsageConfig import preferredTimerPath
from Components.DataBaseAPI import moviedb
from Screens.Screen import Screen
from Screens.ChoiceBox import ChoiceBox
from Screens.EventView import EventViewSimple
from Screens.TimeDateInput import TimeDateInput
from Screens.TimerEntry import TimerEntry
from Screens.EpgSelection import EPGSelection
from Screens.FileDirBrowser import FileDirBrowser
from Screens.TimerEdit import TimerSanityConflict, TimerEditList
from Screens.MessageBox import MessageBox
from Screens.HelpMenu import HelpableScreen
from Tools.Directories import resolveFilename, SCOPE_CURRENT_SKIN, SCOPE_PLUGINS, fileExists
from RecordTimer import RecordTimerEntry, parseEvent, AFTEREVENT
from ServiceReference import ServiceReference
from Tools.LoadPixmap import LoadPixmap
import Tools.AutoTimerHook as AutoTimerHook
from enigma import eServiceReference, eEPGCache, eListbox, eLabel, gFont, eListboxPythonMultiContent, RT_HALIGN_LEFT, RT_HALIGN_CENTER, RT_VALIGN_CENTER, RT_VALIGN_TOP, RT_WRAP, BT_SCALE, BT_FIXRATIO, eRect, eTimer
from time import localtime, time, strftime
from Tools.Alternatives import GetWithAlternative
from Components.Sources.ExtEvent import ExtEvent

import NavigationInstance

import os
import skin

pname = _("Graphical Multi EPG")
pdesc = _("A graphical EPG for all services of an specific bouquet")
pversion = "1.8-r3"
pdate = "20190501"

config.misc.graph_mepg_prev_time=ConfigClock(default = time())
config.misc.graph_mepg_bars = ConfigYesNo(default = False)
config.misc.graph_mepg_prev_time_period=ConfigSelection(default="120", choices = ["60","120","180","240","300","360"])
config.misc.graph_mepg_prev_time_jump=ConfigSelection(default="2", choices = ["1","2","3","4","5","6","7","8","9","10","11","12","13","14","15","16","17","18","19","20","21","22","23","24"])
config.misc.graph_mepg_button_ok = ConfigSelection(default = "2", choices = [("1", _("show single service EPG...")),("2", _("Show detailed programm info")),("3", _("Zap")),("4", _("Preview"))])
config.misc.graph_mepg_button_epg = ConfigSelection(default = "1", choices = [("1", _("show single service EPG...")),("2", _("Show detailed programm info")),("3", _("Zap")),("4", _("Preview"))])
config.misc.graph_mepg_button_yb = ConfigYesNo(default = False)
config.misc.graph_mepg_skin = ConfigText(default = "SKIN0")
config.misc.graph_mepg_picon_dir = ConfigDirectory(default = config.usage.picon_dir.value)
config.misc.graph_mepg_picons = ConfigYesNo(default = True)
config.misc.graph_mepg_zoom = ConfigYesNo(default = True)
config.misc.graph_mepg_flat = ConfigYesNo(default = True)
config.misc.graph_mepg_fontsize = ConfigSelection(default="0", choices = ["-5","-4","-3","-2","-1","0","+1","+2","+3","+4","+5"])
config.misc.graph_mepg_autozap = ConfigYesNo(default = False)

class EPGList(HTMLComponent, GUIComponent):
	searchPiconPaths = [config.misc.graph_mepg_picon_dir.value, '/usr/share/enigma2/picon/', '/etc/picon/', '/media/usb/picon/']
	if config.misc.graph_mepg_picon_dir.value != config.usage.picon_dir.value:
		searchPiconPaths.append(config.usage.picon_dir.value)
	
	def __init__(self, selChangedCB=None, timer = None, time_epoch = 120, overjump_empty=False, skinname = None):
		self.cur_event = None
		self.cur_service = None
		self.offs = 0
		self.timer = timer
		self.onSelChanged = [ ]
		self.skinName = skinname
		if selChangedCB is not None:
			self.onSelChanged.append(selChangedCB)
		GUIComponent.__init__(self)
		self.l = eListboxPythonMultiContent()
		self.l.setItemHeight(54);
		self.l.setBuildFunc(self.buildEntry)
		if overjump_empty:
			self.l.setSelectableFunc(self.isSelectable)
		self.epgcache = eEPGCache.getInstance()
		self.clock_pixmap = LoadPixmap(cached=True, path=resolveFilename(SCOPE_CURRENT_SKIN, 'skin_default/icons/epgclock.png'))
		self.clock_add_pixmap = LoadPixmap(cached=True, path=resolveFilename(SCOPE_CURRENT_SKIN, 'skin_default/icons/epgclock_add.png'))
		self.clock_pre_pixmap = LoadPixmap(cached=True, path=resolveFilename(SCOPE_CURRENT_SKIN, 'skin_default/icons/epgclock_pre.png'))
		self.clock_post_pixmap = LoadPixmap(cached=True, path=resolveFilename(SCOPE_CURRENT_SKIN, 'skin_default/icons/epgclock_post.png'))
		self.clock_prepost_pixmap = LoadPixmap(cached=True, path=resolveFilename(SCOPE_CURRENT_SKIN, 'skin_default/icons/epgclock_prepost.png'))
		self.clock_pixmap_disabled = LoadPixmap(cached=True, path=resolveFilename(SCOPE_CURRENT_SKIN, 'skin_default/icons/epgclock_disabled.png'))
		self.movie_available_pixmap = LoadPixmap(cached=True, path=resolveFilename(SCOPE_CURRENT_SKIN, 'skin_default/icons/movie_is_recorded.png'))
		self.movie_available_and_rec_pixmap = LoadPixmap(cached=True, path=resolveFilename(SCOPE_CURRENT_SKIN, 'skin_default/icons/movie_is_and_will_be_recorded.png'))
		self.movie_available_and_rec_pre_pixmap = LoadPixmap(cached=True, path=resolveFilename(SCOPE_CURRENT_SKIN, 'skin_default/icons/movie_is_and_will_be_recorded_pre.png'))
		self.movie_available_and_rec_post_pixmap = LoadPixmap(cached=True, path=resolveFilename(SCOPE_CURRENT_SKIN, 'skin_default/icons/movie_is_and_will_be_recorded_post.png'))
		self.event_pixmap = None
		self.event_pixmap = LoadPixmap(cached=True, path=resolveFilename(SCOPE_CURRENT_SKIN, 'skin_default/event_back.png'))
		if self.event_pixmap == None:
			self.event_pixmap = LoadPixmap(cached=True, path=resolveFilename(SCOPE_PLUGINS,'Extensions/GraphMultiEPG/pic/event_back.png'))
		self.event_pixmapcur = None
		self.event_pixmapcur = LoadPixmap(cached=True, path=resolveFilename(SCOPE_CURRENT_SKIN, 'skin_default/event_cur.png'))
		if self.event_pixmapcur == None:
			self.event_pixmapcur = LoadPixmap(cached=True, path=resolveFilename(SCOPE_PLUGINS,'Extensions/GraphMultiEPG/pic/event_cur.png'))	
		self.time_base = None
		self.time_epoch = time_epoch
		self.list = None
		self.event_rect = None

		self.test_label = None

		self.nowForeColor = 0xffffff
		self.nowForeColorFlat = 0xffffff
		self.nowForeColorSelected = 0xffff00
		self.nowForeColorSelectedFlat = 0x000000
		
		self.foreColor = None
		self.foreColorFlat = None
		self.foreColorSelected = 0xffff00
		self.foreColorSelectedFlat = 0xffff00
		self.borderColor = None
		self.backColor = 0x586d88
		self.backColorSelected = 0x586d88
		self.nowBackColor = 0x008d68
		self.nowBackColorSelected = 0xd0dda8
		self.foreColorService = None
		self.backColorService = None
		self.recColor = 0xff0000
		self.existrecColor = 0x2f47fe
		self.existandrecColor = 0xaa00ff
		self.prepostColor = 0xffff00
		self.existprepostColor = 0x2673ec
		self.recincompleteColor = 0xff8800
		self.recOffColor = 0x999999
		self.piconWidth = 100
		self.nopiconWidth = 250
		self.piconColor = None
		self.piconColorSelected = None		
		self.valign = "center"
		self.halign = "center"
		self.Font0 = 20
		self.Font1 = 18

	def applySkin(self, desktop, screen):
		if self.skinAttributes is not None:
			attribs = [ ]
			for (attrib, value) in self.skinAttributes:
				if attrib == "EntryForegroundColor":
					self.foreColor = parseColor(value).argb()
				elif attrib == "EntryForegroundColorFlat": 					# New for skin
					self.foreColorFlat = parseColor(value).argb()
				elif attrib == "EntryNowForegroundColor":
					self.nowForeColor = parseColor(value).argb()
				elif attrib == "EntryNowForegroundColorFlat": 				# New for skin
					self.nowForeColorFlat = parseColor(value).argb()				
					
				elif attrib == "EntryForegroundColorSelected":				# keep for skin compatibility
					self.foreColorSelectedFlat = parseColor(value).argb()
				elif attrib == "EntryForegroundColorSelectedFlat":			# keep for skin compatibility
					self.foreColorSelected = parseColor(value).argb()
				elif attrib == "EntryNowForegroundColorSelected":			# keep for skin compatibility
					self.nowForeColorSelectedFlat = parseColor(value).argb()
				elif attrib == "EntryNowForegroundColorSelectedFlat":		# keep for skin compatibility
					self.nowForeColorSelected = parseColor(value).argb()

				elif attrib == "EntryForegroundColorSel": 					# New for skin
					self.foreColorSelected = parseColor(value).argb()
				elif attrib == "EntryForegroundColorSelFlat": 				# New for skin
					self.foreColorSelectedFlat = parseColor(value).argb()					
				elif attrib == "EntryNowForegroundColorSel": 				# New for skin
					self.nowForeColorSelected = parseColor(value).argb()
				elif attrib == "EntryNowForegroundColorSelFlat": 			# New for skin
					self.nowForeColorSelectedFlat = parseColor(value).argb()
					
				elif attrib == "EntryBorderColor":
					self.borderColor = parseColor(value).argb()
				elif attrib == "EntryBackgroundColor":
					self.backColor = parseColor(value).argb()
				elif attrib == "EntryNowBackgroundColor":
					self.nowBackColor = parseColor(value).argb()
				elif attrib == "EntryBackgroundColorSelected":
					self.backColorSelected = parseColor(value).argb()
				elif attrib == "EntryNowBackgroundColorSelected":
					self.nowBackColorSelected = parseColor(value).argb()
				elif attrib == "ServiceNameForegroundColor":
					self.foreColorService = parseColor(value).argb()
				elif attrib == "ServiceNameBackgroundColor":
					self.backColorService = parseColor(value).argb()
				elif attrib == "EntryRecColor":
					self.recColor = parseColor(value).argb()
				elif attrib == "ExistEntryRecColor":
					self.existrecColor = parseColor(value).argb()					
				elif attrib == "ExistAndRecEntryRecColor":
					self.existandrecColor = parseColor(value).argb()							
				elif attrib == "EntryPreColor":
					self.prepostColor = parseColor(value).argb()
				elif attrib == "ExistEntryPreColor":
					self.existprepostColor = parseColor(value).argb()
				elif attrib == "EntryRecIncompleteColor":
					self.recincompleteColor = parseColor(value).argb()					
				elif attrib == "EntryRecOffColor":
					self.recOffColor = parseColor(value).argb()
				elif attrib == "piconWidth":
					self.piconWidth = int(value)
				elif attrib == "nopiconWidth":
					self.nopiconWidth = int(value)
				elif attrib == "piconColor":
					self.piconColor = parseColor(value).argb()
				elif attrib == "piconColorSelected":
					self.piconColorSelected = parseColor(value).argb()					
				elif attrib == "valign":
					self.valign = str(value)
				elif attrib == "halign":
					self.halign = str(value)
				elif attrib == "FontSize1":
					self.Font0 = int(value)
				elif attrib == "FontSize2":
					self.Font1 = int(value)
				else:
					attribs.append((attrib,value))
			self.skinAttributes = attribs

		df = int(config.misc.graph_mepg_fontsize.value)
		self.l.setFont(0, gFont("Regular", self.Font0 + df))
		self.l.setFont(1, gFont("Regular", self.Font1 + df))
		self.txfont = gFont("Regular", self.Font1 + df)
		self.test_label.setFont(self.txfont)
		return GUIComponent.applySkin(self, desktop, screen)

	def isSelectable(self, service, sname, event_list):
		return (event_list and len(event_list) and True) or False

	def setEpoch(self, epoch):
		self.offs = 0
		self.time_epoch = epoch
		self.fillMultiEPG(None)

	def getEventFromId(self, service, eventid):
		event = None
		if self.epgcache is not None and eventid is not None:
			event = self.epgcache.lookupEventId(service.ref, eventid)
		return event

	def moveToService(self,serviceref):
		if serviceref is not None:
			for x in range(len(self.list)):
				if self.list[x][0] == serviceref.toString():
					self.instance.moveSelectionTo(x)
					break
	
	def getIndexFromService(self, serviceref):
		if serviceref is not None:
			for x in range(len(self.list)):
				if self.list[x][0] == serviceref.toString():
					return x
		
	def setCurrentIndex(self, index):
		if self.instance is not None:
			self.instance.moveSelectionTo(index)

	def getCurrentChangeCount(self):
		if self.l.getCurrentSelection() is not None:
			return self.l.getCurrentSelection()
		return 0
			
	def getCurrent(self):
		if self.cur_service is None:
			return ( None, None )
		old_service = self.cur_service
		events = self.cur_service[2]
		refstr = self.cur_service[0]
		if self.cur_event is None or not events or not len(events):
			return ( None, ServiceReference(refstr) )
		event = events[self.cur_event]
		eventid = event[0]
		service = ServiceReference(refstr)
		event = self.getEventFromId(service, eventid)
		return ( event, service )

	def connectSelectionChanged(func):
		if not self.onSelChanged.count(func):
			self.onSelChanged.append(func)

	def disconnectSelectionChanged(func):
		self.onSelChanged.remove(func)

	def serviceChanged(self):
		cur_sel = self.l.getCurrentSelection()
		if cur_sel:
			self.findBestEvent()

	def findBestEvent(self):
		old_service = self.cur_service
		cur_service = self.cur_service = self.l.getCurrentSelection()
		last_time = 0;
		time_base = self.getTimeBase()
		if old_service and self.cur_event is not None:
			events = old_service[2]
			cur_event = events[self.cur_event]
			last_time = cur_event[2]
			if last_time < time_base:
				last_time = time_base
		if cur_service:
			self.cur_event = 0
			events = cur_service[2]
			if events and len(events):
				if last_time:
					best_diff = 0
					best = len(events)
					idx = 0
					for event in events:
						ev_time = event[2]
						if ev_time < time_base:
							ev_time = time_base
						diff = abs(ev_time-last_time)
						if (best == len(events)) or (diff < best_diff):
							best = idx
							best_diff = diff
						end_event = event[2]+event[3]
						if end_event < int(time()) and ev_time == time_base:
							best = best + 1
						idx += 1
					if best != len(events):
						self.cur_event = best
			else:
				self.cur_event = None
		self.selEntry(0)

	def selectionChanged(self):
		for x in self.onSelChanged:
			if x is not None:
				x()

	GUI_WIDGET = eListbox

	def postWidgetCreate(self, instance):
		instance.setWrapAround(True)
		instance.selectionChanged.get().append(self.serviceChanged)
		instance.setContent(self.l)
		self.test_label = eLabel(instance)

		df = int(config.misc.graph_mepg_fontsize.value)
		self.l.setFont(0, gFont("Regular", self.Font0 + df))
		self.l.setFont(1, gFont("Regular", self.Font1 + df))
		self.txfont = gFont("Regular", self.Font1 + df)

		self.test_label.setFont(self.txfont)
		self.l.setSelectionClip(eRect(0,0,0,0), False)

	def preWidgetRemove(self, instance):
		instance.selectionChanged.get().remove(self.serviceChanged)
		instance.setContent(None)
		self.test_label = None

	def reload(self):
		self.l.setList(self.list)

	def recalcEntrySize(self):
		esize = self.l.getItemSize()
		if config.misc.graph_mepg_picons.value:
			pzize = self.piconWidth
		else:
			pzize = self.nopiconWidth
		width = esize.width()
		height = esize.height()
		xpos = 0;
		w = pzize;
		self.service_rect = Rect(xpos, 0, w, height)
		xpos += w;
		w = width - xpos;
		self.event_rect = Rect(xpos, 0, w, height)

	def calcEntryPosAndWidthHelper(self, stime, duration, start, end, width):
		xpos = (stime - start) * width / (end - start)
		ewidth = (stime + duration - start) * width / (end - start)
		ewidth -= xpos;
		if xpos < 0:
			ewidth += xpos;
			xpos = 0;
		if (xpos+ewidth) > width:
			ewidth = width - xpos
		return xpos, ewidth

	def calcEntryPosAndWidth(self, event_rect, time_base, time_epoch, ev_start, ev_duration):
		xpos, width = self.calcEntryPosAndWidthHelper(ev_start, ev_duration, time_base, time_base + time_epoch * 60, event_rect.width())
		return xpos+event_rect.left(), width

	def buildEntry(self, service, service_name, events):
		if service == self.cur_service[0]:
			piconbkcolor = self.piconColorSelected
			serviceFGcolorCurService = self.foreColorSelectedFlat
			serviceBGcolorCurService = self.backColorSelected
		else:
			piconbkcolor = self.piconColor
			serviceFGcolorCurService = self.foreColorService
			serviceBGcolorCurService = self.backColorService
			
		r1=self.service_rect
		r2=self.event_rect
		res = [ None ]
		
		dx,dy,wx,hy = skin.parameters.get("GMultiEPGClock", (0,0,21,21))	
		
		picon = self.findPicon(service, service_name)

		scale_picon = config.usage.picon_scale.value
		if picon is None:
			res.append(MultiContentEntryText(
				pos = (r1.left(),r1.top()),
				size = (r1.width(), r1.height()),
				font = 0, flags = RT_HALIGN_LEFT | RT_VALIGN_CENTER,
				text = service_name,
				color = serviceFGcolorCurService,
				backcolor = serviceBGcolorCurService,
				backcolor_sel = 0))
		else:
			res.append(MultiContentEntryPixmapAlphaBlend(
				pos = (r1.left()+1,r1.top()+1),
				size = (r1.width()-2, r1.height()-2),
				png = LoadPixmap(picon),
				backcolor = piconbkcolor,
				backcolor_sel = 0,
				options = BT_SCALE | BT_FIXRATIO))

		if events:
			start = self.time_base+self.offs*self.time_epoch*60
			end = start + self.time_epoch * 60
			left = r2.left()
			top = r2.top()
			width = r2.width()
			height = r2.height()
			borderColor = self.borderColor
			now = int(time())
			for ev in events:
				movie_rec = 0
				rec=ev[2] and self.timer.isInTimer(ev[0], ev[2], ev[3], service)
				if int(config.misc.timer_show_movie_available.value) > 0 and ev[1] is not None:
					shortDesc = ''
					extDesc = ''
					if int(config.misc.timer_show_movie_available.value) > 1:
						serviceref = ServiceReference(service)
						tmp_evt = self.epgcache.lookupEventId(serviceref.ref, ev[0])
						if tmp_evt is not None:
							shortDesc = tmp_evt.getShortDescription()
							extDesc = tmp_evt.getExtendedDescription()
					if moviedb.inTitleList(ev[1], shortDesc, extDesc) is not None:
						movie_rec = 1
				rec_inactive = False
				for timerOff in self.timer.processed_timers:
					if timerOff.eit == ev[0]:
						rec_inactive = True
				xpos, ewidth = self.calcEntryPosAndWidthHelper(ev[2], ev[3], start, end, width)
				title = str(ev[1])
				text_size = self.test_label.calculateSize()
				text_height = text_size.height()
				if ev[2] <= now and (ev[2] + ev[3]) > now:
					foreColor = self.nowForeColorFlat
					foreColorSelected = self.nowForeColorSelectedFlat
					backColor = self.nowBackColor
					backColorSelected = self.nowBackColorSelected
					if not config.misc.graph_mepg_flat.value:
						cur_png = self.event_pixmapcur
						foreColor = self.nowForeColor
						foreColorSelected = self.nowForeColorSelected
						backColor = None
						backColorSelected = None
				else:
					foreColor = self.foreColorFlat
					foreColorSelected = self.foreColorSelectedFlat
					backColor = self.backColor
					backColorSelected = self.backColorSelected
					if not config.misc.graph_mepg_flat.value:
						cur_png = self.event_pixmap
						foreColor = self.foreColor
						foreColorSelected = self.foreColorSelected
						backColor = None
						backColorSelected = None
					
				if ewidth > 1:
					if height > 49 and height < 75 or height > 49 and ewidth < 180:
						self.test_label.setText(ev[1])
						text_size = self.test_label.calculateSize()
						text_height = text_size.height()
						if (text_height*0.8/ewidth) > 1:
							my_valign = RT_VALIGN_TOP
						else:
							my_valign = RT_VALIGN_CENTER
							
						if not config.misc.graph_mepg_flat.value:
							res.append(MultiContentEntryPixmapAlphaBlend(
								pos = (left+xpos, top), size = (ewidth, height),
								png = cur_png, options = BT_SCALE))
							
						res.append(MultiContentEntryText(
							pos = (left+xpos, top), size = (ewidth, height),
							font = 1, flags = RT_HALIGN_CENTER | my_valign | RT_WRAP,
							text = ev[1], color = foreColor, color_sel = foreColorSelected,
							backcolor = backColor, backcolor_sel = backColorSelected, border_width = 1, border_color = borderColor))
					elif height >= 75 and ewidth >= 180:
						my_valign = RT_VALIGN_CENTER
						my_halign = RT_HALIGN_LEFT
						self.test_label.setNoWrap(1)
						self.test_label.setText(str(title))
						text_size = self.test_label.calculateSize()
						try:
							if text_size.width() > ewidth-10:
								resizing = True
								i = -1
								while resizing:
									self.test_label.setText("%s%s" % (title[:i], "..."))
									if self.test_label.calculateSize().width() < ewidth-10:
										text_size = self.test_label.calculateSize()
										title = "%s%s" % (title[:i], "...")
										resizing = False
									else:
										i -= 1
						except Exception, ex:
							vtilog("Error Resizing Title: %s" % str(ex))
						eventshort = ''
						serviceref = ServiceReference(service)
						eventshort = self.epgcache.lookupEventId(serviceref.ref, ev[0])
						eventshort = eventshort.getShortDescription()
						self.test_label.setNoWrap(1)
						self.test_label.setText(str(eventshort))
						text_size = self.test_label.calculateSize()
						try:
							if text_size.width() > ewidth-10:
								resizing = True
								i = -1
								while resizing:
									self.test_label.setText("%s%s" % (eventshort[:i], "..."))
									if self.test_label.calculateSize().width() < ewidth-10:
										text_size = self.test_label.calculateSize()
										eventshort = "%s%s" % (eventshort[:i], "...")
										resizing = False

									else:
										i -= 1
						except Exception, ex:
							vtilog("Error Resizing Eventshort: %s" % str(ex)		)
						if title == eventshort:
							eventshort = ''
						t = localtime(ev[2])
						eventtime = "%02d:%02d"%(t[3],t[4])
						
						if not config.misc.graph_mepg_flat.value:
							res.append(MultiContentEntryPixmapAlphaBlend(
								pos = (left+xpos, top), size = (ewidth, height),
								png = cur_png, options = BT_SCALE))	
						
						res.append(MultiContentEntryText(
							pos = (left+xpos, top), size = (ewidth, height),
							font = 1, flags = my_halign | my_valign,
							text = " " + eventtime + "\n " + title + "\n " + eventshort, color = foreColor, color_sel = foreColorSelected,
							backcolor = backColor, backcolor_sel = backColorSelected, border_width = 1, border_color = borderColor))

					else:
						self.test_label.setText(ev[1])
						text_size = self.test_label.calculateSize()
						text_height = text_size.height()
						if (text_height*0.8/ewidth) > 1 or self.halign == "left":
							event_Name = "  " + ev[1]
							my_halign = RT_HALIGN_LEFT
						elif self.halign == "right":
							event_Name = ev[1] + "  "
							my_halign = RT_HALIGN_RIGHT
						else:
							event_Name = ev[1]
							my_halign = RT_HALIGN_CENTER
						
						if not config.misc.graph_mepg_flat.value:
							res.append(MultiContentEntryPixmapAlphaBlend(
								pos = (left+xpos, top), size = (ewidth, height),
								png = cur_png, options = BT_SCALE))
						
						res.append(MultiContentEntryText(
							pos = (left+xpos, top), size = (ewidth, height),
							font = 1, flags = my_halign | RT_VALIGN_CENTER,
							text = event_Name, color = foreColor, color_sel = foreColorSelected,
							backcolor = backColor, backcolor_sel = backColorSelected, border_width = 1, border_color = borderColor))
				else:
					if not config.misc.graph_mepg_flat.value:
						res.append(MultiContentEntryPixmapAlphaBlend(
							pos = (left+xpos, top), size = (ewidth, height),
							png = cur_png, options = BT_SCALE))
				
					res.append(MultiContentEntryText(
					pos = (left+xpos, top), size = (ewidth, height),
					font = 1, flags = RT_HALIGN_CENTER | RT_VALIGN_CENTER | RT_WRAP,
					text = ev[1], color = foreColor, color_sel = foreColorSelected,
					backcolor = backColor, backcolor_sel = backColorSelected, border_width = 1, border_color = borderColor))
					
				if config.misc.graph_mepg_bars.value:
					if rec_inactive and ewidth > 23:
						backColor = self.recOffColor
						backColorSelected = self.recOffColor
						res.append(MultiContentEntryText(
							pos = (left+xpos, top+height-7), size = (ewidth, 6),
							font = 1, flags = RT_HALIGN_CENTER | RT_VALIGN_CENTER | RT_WRAP,
							text = "", color = foreColor, color_sel = foreColorSelected,
							backcolor = backColor, backcolor_sel = backColorSelected, border_width = 0, border_color = borderColor))
					if (rec or movie_rec) and ewidth > 23:
						prefac, postfac, prerecpost = self.getBars(service, ev[2], ev[3], ev[0], ev[1], is_movie_rec = movie_rec)
						foreColor = self.nowForeColorFlat
						foreColorSelected = self.nowForeColorSelectedFlat
						if not config.misc.graph_mepg_flat.value:
							foreColor = self.nowForeColor
							foreColorSelected = self.nowForeColorSelected
						backColor = self.prepostColor
						backColorSelected = self.prepostColor
						if prerecpost == 0:
							backColor = self.recColor
							backColorSelected = self.recColor
							res.append(MultiContentEntryText(
								pos = (left+xpos, top+height-7), size = (ewidth, 6),
								font = 1, flags = RT_HALIGN_CENTER | RT_VALIGN_CENTER | RT_WRAP,
								text = "", color = foreColor, color_sel = foreColorSelected,
								backcolor = backColor, backcolor_sel = backColorSelected, border_width = 0, border_color = borderColor))
						if prerecpost == 9:
							backColor = self.existrecColor
							backColorSelected = self.existrecColor
							res.append(MultiContentEntryText(
								pos = (left+xpos, top+height-7), size = (ewidth, 6),
								font = 1, flags = RT_HALIGN_CENTER | RT_VALIGN_CENTER | RT_WRAP,
								text = "", color = foreColor, color_sel = foreColorSelected,
								backcolor = backColor, backcolor_sel = backColorSelected, border_width = 0, border_color = borderColor))
						if prerecpost == 8:
							backColor = self.existandrecColor
							backColorSelected = self.existandrecColor
							res.append(MultiContentEntryText(
								pos = (left+xpos, top+height-7), size = (ewidth, 6),
								font = 1, flags = RT_HALIGN_CENTER | RT_VALIGN_CENTER | RT_WRAP,
								text = "", color = foreColor, color_sel = foreColorSelected,
								backcolor = backColor, backcolor_sel = backColorSelected, border_width = 0, border_color = borderColor))
						if prerecpost == 4:
							backColor = self.recincompleteColor
							backColorSelected = self.recincompleteColor
							res.append(MultiContentEntryText(
								pos = (left+xpos+(ewidth*prefac), top+height-7), size = (ewidth*postfac, 6),
								font = 1, flags = RT_HALIGN_CENTER | RT_VALIGN_CENTER | RT_WRAP,
								text = "", color = foreColor, color_sel = foreColorSelected,
								backcolor = backColor, backcolor_sel = backColorSelected, border_width = 0, border_color = borderColor))
						if prerecpost == 5:
							backColor = self.recincompleteColor
							backColorSelected = self.recincompleteColor
							res.append(MultiContentEntryText(
								pos = (left+xpos+(ewidth*prefac), top+height-7), size = (ewidth-(ewidth*prefac), 6),
								font = 1, flags = RT_HALIGN_CENTER | RT_VALIGN_CENTER | RT_WRAP,
								text = "", color = foreColor, color_sel = foreColorSelected,
								backcolor = backColor, backcolor_sel = backColorSelected, border_width = 0, border_color = borderColor))
						elif prerecpost == 1:
							res.append(MultiContentEntryText(
								pos = (left+xpos+(ewidth-ewidth*prefac), top+height-7), size = (ewidth*prefac, 6),
								font = 1, flags = RT_HALIGN_CENTER | RT_VALIGN_CENTER | RT_WRAP,
								text = "", color = foreColor, color_sel = foreColorSelected,
								backcolor = backColor, backcolor_sel = backColorSelected, border_width = 0, border_color = borderColor))
						
						elif prerecpost == 7:
							backColor = self.existprepostColor
							backColorSelected = self.existprepostColor	
							res.append(MultiContentEntryText(
								pos = (left+xpos+(ewidth-ewidth*prefac), top+height-7), size = (ewidth*prefac, 6),
								font = 1, flags = RT_HALIGN_CENTER | RT_VALIGN_CENTER | RT_WRAP,
								text = "", color = foreColor, color_sel = foreColorSelected,
								backcolor = backColor, backcolor_sel = backColorSelected, border_width = 0, border_color = borderColor))
								
						elif prerecpost == 2:
							res.append(MultiContentEntryText(
								pos = (left+xpos, top+height-7), size = (ewidth*prefac, 6),
								font = 1, flags = RT_HALIGN_CENTER | RT_VALIGN_CENTER | RT_WRAP,
								text = "", color = foreColor, color_sel = foreColorSelected,
								backcolor = backColor, backcolor_sel = backColorSelected, border_width = 0, border_color = borderColor))

						elif prerecpost == 6:
							backColor = self.existprepostColor
							backColorSelected = self.existprepostColor						
							res.append(MultiContentEntryText(
								pos = (left+xpos, top+height-7), size = (ewidth*prefac, 6),
								font = 1, flags = RT_HALIGN_CENTER | RT_VALIGN_CENTER | RT_WRAP,
								text = "", color = foreColor, color_sel = foreColorSelected,
								backcolor = backColor, backcolor_sel = backColorSelected, border_width = 0, border_color = borderColor))
								
						elif prerecpost == 3:
							res.append(MultiContentEntryText(
								pos = (left+xpos, top+height-7), size = (ewidth*prefac, 6),
								font = 1, flags = RT_HALIGN_CENTER | RT_VALIGN_CENTER | RT_WRAP,
								text = "", color = foreColor, color_sel = foreColorSelected,
								backcolor = backColor, backcolor_sel = backColorSelected, border_width = 0, border_color = borderColor))
				else:
					if (rec or movie_rec) and ewidth > 23:
						res.append(MultiContentEntryPixmapAlphaTest(
							pos = (left+xpos+ewidth-22+dx, top+height-22+dy), size = (wx, hy),
							png = self.getClockPixmap(service, ev[2], ev[3], ev[0], is_movie_rec = movie_rec),
							backcolor = backColor,
							backcolor_sel = backColorSelected))
					if rec_inactive and ewidth > 23:
						clock_pic = self.clock_pixmap_disabled
						res.append(MultiContentEntryPixmapAlphaTest(
							pos = (left+xpos+ewidth-22+dx, top+height-22+dy), size = (wx, hy),
							png = clock_pic,
							backcolor = backColor,
							backcolor_sel = backColorSelected))
		return res

	def findPicon(self, service = None, serviceName = None):
		if config.misc.graph_mepg_picons.value == False:
			pngname = None
			return pngname
		if service is not None:
			pos = service.rfind(':')
			if pos != -1:
				if service.startswith("1:134"):
					service = GetWithAlternative(service)
					pos = service.rfind(':')
				service = service[:pos].rstrip(':').replace(':','_')
			pos = service.rfind('_http')
			if pos != -1:
					service = service[:pos].rstrip('_http').replace(':','_')
			pngname = "/tmp/gmepgpicon/" + service + ".png"
			if fileExists(pngname):
				return pngname
		if serviceName is not None:
			pngname = "/tmp/gmepgpicon/" + serviceName + ".png"
			if fileExists(pngname):
				return pngname

		if service is not None:
			for path in self.searchPiconPaths:
				if path[-1] != '/':
					path = path + '/'
				pngname = path + service + ".png"
				if fileExists(pngname):
					return pngname
		if serviceName is not None:
			for path in self.searchPiconPaths:
				if path[-1] != '/':
					path = path + '/'
				pngname = path + serviceName + ".png"
				if fileExists(pngname):
					return pngname
	
	def selEntry(self, dir, visible=True):
		cur_service = self.cur_service
		self.recalcEntrySize()
		valid_event = self.cur_event is not None
		if cur_service:
			update = True
			entries = cur_service[2]
			if dir == 0:
				update = False
			elif dir == +1:
				if valid_event and self.cur_event+1 < len(entries):
					self.cur_event+=1
				else:
					self.offs += 1
					self.fillMultiEPG(None)
					return True
			elif dir == -1:
				if valid_event and self.cur_event-1 >= 0:
					self.cur_event-=1
				elif valid_event:
					self.offs -= 1
					self.fillMultiEPG(None)
					return True
		self.l.setSelectionClip(eRect(self.service_rect.left(), self.service_rect.top(), self.service_rect.width(), self.service_rect.height()), False)
		if cur_service and valid_event:
			entry = entries[self.cur_event]
			time_base = self.time_base+self.offs*self.time_epoch*60
			xpos, width = self.calcEntryPosAndWidth(self.event_rect, time_base, self.time_epoch, entry[2], entry[3])
			self.l.setSelectionClip(eRect(xpos, 0, width, self.event_rect.height()), visible and update)
		else:
			self.l.setSelectionClip(eRect(self.event_rect.left(), self.event_rect.top(), self.event_rect.width(), self.event_rect.height()), False)
		self.selectionChanged()
		return False

	def getEntry(self):
		service_row = 0
		event_left = 0
		event_width = 0
		service_height = 0
		cur_service = self.cur_service
		valid_event = self.cur_event is not None
		entries = cur_service[2]
		if cur_service and valid_event:
			entry = entries[self.cur_event]
			time_base = self.time_base+self.offs*self.time_epoch*60
			xpos, width = self.calcEntryPosAndWidth(self.event_rect, time_base, self.time_epoch, entry[2], entry[3])
			event_left = xpos
			event_width = width
		return event_left,event_width
		
	def queryEPG(self, list, buildFunc=None):
		if self.epgcache is not None:
			if buildFunc is not None:
				return self.epgcache.lookupEvent(list, buildFunc)
			else:
				return self.epgcache.lookupEvent(list)
		return [ ]

	def fillMultiEPG(self, services, stime=-1):
		if services is None:
			time_base = self.time_base+self.offs*self.time_epoch*60
			test = [ (service[0], 0, time_base, self.time_epoch) for service in self.list ]
		else:
			self.cur_event = None
			self.cur_service = None
			self.time_base = int(stime)
			test = [ (service.ref.toString(), 0, self.time_base, self.time_epoch) for service in services ]
		test.insert(0, 'XRNITBD')
		epg_data = self.queryEPG(test)
		self.list = [ ]
		tmp_list = None
		service = ""
		sname = ""
		for x in epg_data:
			if service != x[0]:
				if tmp_list is not None:
					self.list.append((service, sname, tmp_list[0][0] is not None and tmp_list or None))
				service = x[0]
				sname = x[1]
				tmp_list = [ ]
			tmp_list.append((x[2], x[3], x[4], x[5]))
		if tmp_list and len(tmp_list):
			self.list.append((service, sname, tmp_list[0][0] is not None and tmp_list or None))

		self.l.setList(self.list)
		self.findBestEvent()

	def getEventRect(self):
		rc = self.event_rect
		return Rect( rc.left() + (self.instance and self.instance.position().x() or 0), rc.top(), rc.width(), rc.height() )

	def getTimeEpoch(self):
		return self.time_epoch

	def getTimeBase(self):
		return self.time_base + (self.offs * self.time_epoch * 60)

	def resetOffset(self):
		self.offs = 0

	def getBars(self, refstr, beginTime, duration, eventId, eventName, is_movie_rec = False):
		prefac = 1
		postfac = 1
		pre_bar = 1
		post_bar = 2
		bar_type = 0
		endTime = beginTime + duration
		for x in self.timer.timer_list:
			if x.service_ref.ref.toString() == refstr:
				begTimer = x.begin
				endTimer = x.end
				if x.eit == eventId:
					if begTimer > beginTime and endTimer < endTime:
						prefac = (begTimer - beginTime) / float(duration)
						postfac = (endTimer - begTimer) / float(duration)
						return prefac, postfac, 4
					elif begTimer < beginTime and endTimer < endTime:
						prefac = 0
						postfac = (endTimer - beginTime) / float(duration)
						return prefac, postfac, 4
					elif begTimer > beginTime and endTimer > endTime:
						prefac = (begTimer - beginTime) / float(duration)
						postfac = 0
						return prefac, postfac, 5
					if is_movie_rec:
						return prefac, postfac, 8
					else:
						return prefac, postfac, 0
				if beginTime > begTimer and beginTime < endTimer and endTime > endTimer:
					endTimerPre = x.end
					bar_type |= pre_bar
				elif beginTime < begTimer and endTime > begTimer and endTime < endTimer:
					begTimerPost = x.begin
					bar_type |= post_bar
		if bar_type == 0:
			if is_movie_rec:
				return prefac, postfac, 9
			else:
				return prefac, postfac, 0
		elif bar_type == pre_bar:
			prefac = (endTimerPre - beginTime) / float(duration)
			if is_movie_rec:
				return prefac, postfac, 6
			else:
				return prefac, postfac, 2
		elif bar_type == post_bar:
			prefac = (endTime - begTimerPost) / float(duration)
			if is_movie_rec:
				return prefac, postfac, 7
			else:
				return prefac, postfac, 1
		else:
			prefac = 1
			return prefac, postfac, 3

	def getClockPixmap(self, refstr, beginTime, duration, eventId, is_movie_rec = False):
		fac = 1
		pre_clock = 1
		post_clock = 2
		clock_type = 0
		endTime = beginTime + duration
		for x in self.timer.timer_list:
			if x.service_ref.ref.toString() == refstr:
				if x.eit == eventId:
					if is_movie_rec:
						return self.movie_available_and_rec_pixmap
					else:
						return self.clock_pixmap
				beg = x.begin
				end = x.end
				if beginTime > beg and beginTime < end and endTime > end:
					clock_type |= pre_clock
				elif beginTime < beg and endTime > beg and endTime < end:
					clock_type |= post_clock
		if clock_type == 0 and not is_movie_rec:
			return self.clock_add_pixmap
		elif clock_type == 0 and is_movie_rec:
			return self.movie_available_pixmap
		elif clock_type == pre_clock:
			fac = (float(end) - float(beginTime)) / float(duration)
			if is_movie_rec:
				return self.movie_available_and_rec_pre_pixmap
			else:
				return self.clock_pre_pixmap
		elif clock_type == post_clock:
			fac = (float(endTime) - float(beg)) / float(duration)
			if is_movie_rec:
				return self.movie_available_and_rec_post_pixmap
			else:
				return self.clock_post_pixmap
		else:
			return self.clock_prepost_pixmap

class TimelineText(HTMLComponent, GUIComponent):
	def __init__(self):
		GUIComponent.__init__(self)
		self.l = eListboxPythonMultiContent()
		self.l.setSelectionClip(eRect(0,0,0,0))
		self.l.setItemHeight(25);
		font = skin.fonts.get("GMultiEPGTimelineFont", ("Regular", 20))
		self.l.setFont(0, gFont(font[0], font[1]))

	GUI_WIDGET = eListbox

	def postWidgetCreate(self, instance):
		instance.setContent(self.l)

	def setEntries(self, entries):
		res = [ None ]
		for x in entries:
			tm = x[0]
			xpos = x[1]
			str = strftime("%H:%M", localtime(tm))
			w,h = skin.parameters.get("GMultiEPGTimelineItemSize", (60,25))
			res.append((eListboxPythonMultiContent.TYPE_TEXT, xpos-w/2, 0, w, h, 0, RT_HALIGN_CENTER|RT_VALIGN_CENTER, str))
		self.l.setList([res])

class GraphMultiEPGConfigScreen(Screen, ConfigListScreen):
	def __init__(self, session):
		Screen.__init__(self, session)
		self.skinName = [ "GraphMultiEPGConfigScreen", "Setup" ]
		self.setup_title = _("GraphMultiEPG setup")

		self.onChangedEntry = [ ]
		self.list = [ ]
		ConfigListScreen.__init__(self, self.list, session = session, on_change = self.changedEntry)

		self["actions"] = ActionMap(["ColorActions", "SetupActions"],
			{
				"cancel": self.keyCancel,
				"save": self.keyOK,
				"red": self.keyCancel,
				"green": self.keyGreen,
				"left": self.myKeyLeft,
				"right": self.myKeyRight,
			}, -2)

		self["key_green"] = StaticText(_("OK"))
		self["key_red"] = StaticText(_("Cancel"))

		self.list = []
		self.createConfigList()	
		self.onLayoutFinish.append(self.layoutFinished)

	def layoutFinished(self):
		self.setTitle(pname + " (" + pversion + ")")

	def createConfigList(self):
		self.list = []
		self.list.append(getConfigListEntry(_("Enter Prime Time"), config.misc.graph_mepg_prev_time))
		self.list.append(getConfigListEntry(_("Bars view for timer"), config.misc.graph_mepg_bars))
		self.list.append(getConfigListEntry(_("Time frame of entries in minutes"), config.misc.graph_mepg_prev_time_period))
		self.list.append(getConfigListEntry(_("Set jump time in hours"), config.misc.graph_mepg_prev_time_jump))
		self.list.append(getConfigListEntry(_("Ok button action"), config.misc.graph_mepg_button_ok))
		self.list.append(getConfigListEntry(_("EPG button action"), config.misc.graph_mepg_button_epg))
		self.list.append(getConfigListEntry(_("Do you want to change channels with Channel +/- buttons?"), config.misc.graph_mepg_button_yb))
		self.list.append(getConfigListEntry(_("Picon folder for graphical multi EPG"), config.misc.graph_mepg_picon_dir))
		self.list.append(getConfigListEntry(_("Picons"), config.misc.graph_mepg_picons))
		self.list.append(getConfigListEntry(_("Zoom"), config.misc.graph_mepg_zoom))
		self.list.append(getConfigListEntry(_("Flat"), config.misc.graph_mepg_flat))
		self.list.append(getConfigListEntry(_("Font size"), config.misc.graph_mepg_fontsize))
		self.list.append(getConfigListEntry(_("Auto Zap"), config.misc.graph_mepg_autozap))
		self["config"].list = self.list
		self["config"].setList(self.list)

	def changedEntry(self):
		for x in self.onChangedEntry:
			x()

	def keyGreen(self):
		for x in self["config"].list:
			x[1].save()
		self.close()

	def keyOK(self):
		self.keyGreen()

	def myKeyLeft(self):
		cur_config = self["config"].getCurrent()[1]
		if isinstance(cur_config, ConfigDirectory):
			self.selectPath(cur_config)
		else:
			self.keyLeft()

	def myKeyRight(self):
		cur_config = self["config"].getCurrent()[1]
		if isinstance(cur_config, ConfigDirectory):
			self.selectPath(cur_config)
		else:
			self.keyRight()

	def selectPath(self, cur_config):
		if os.path.exists(cur_config.value):
			initDir = cur_config.value
			if not initDir[:-1] == "/":
				initDir += "/"
		else:
			initDir = '/'
		self.session.openWithCallback(self.pathSelected, FileDirBrowser, title = _("Choose folder"), getFile = False, getDir = True, initDir = initDir)

	def pathSelected(self, path):
		if path:
			self["config"].getCurrent()[1].value = path

class GraphMultiEPG(Screen, HelpableScreen):
	EMPTY = 0
	ADD_TIMER = 1
	REMOVE_TIMER = 2
	ENABLE_TIMER = 3
	ZAP = 1
	
	def __init__(self, session, services, zapFunc=None, bouquetChangeCB=None, getbouquetName=None):
		Screen.__init__(self, session)
		if not AutoTimerHook.CHECKAUTOTIMER:
			AutoTimerHook.initAutoTimerGlobals()
		HelpableScreen.__init__(self)

		if config.misc.graph_mepg_skin.value == "SKIN0":
			self.skinName = "GraphMultiEPG"
		if config.misc.graph_mepg_skin.value == "SKIN1":
			self.skinName = "GraphMultiEPG1"
		if config.misc.graph_mepg_skin.value == "SKIN2":
			self.skinName = "GraphMultiEPG2"
		if config.misc.graph_mepg_skin.value == "SKIN3":
			self.skinName = "GraphMultiEPG3"
		
		self.doAutoZap = False
		myscreen, path = lookupScreen(self.skinName, 0)
		for element in myscreen:
			wname = element.attrib.get('render')
			if wname == "Pig":
				self.doAutoZap = True
		cur_ref = NavigationInstance.instance.getCurrentlyPlayingServiceReference()
		if cur_ref.getPath():
			self.doAutoZap = False
		
		self.bouquetChangeCB = bouquetChangeCB
		self.bouquetName = getbouquetName
		
		self["zoom"] = Label()
		self["zoomBack"] = Label()
		self["zoomShadow"] = Label()
		self["zoom"].hide()
		self["zoomBack"].hide()
		self["zoomShadow"].hide
		
		now = time()
		tmp = now % 900
		self.ask_time = now - tmp
		self.closeRecursive = False
		self["key_red"] = Button("")
		self["key_green"] = Button("")
			
		self.key_green_choice = self.EMPTY
		self.key_red_choice = self.EMPTY
		self["timeline_text"] = TimelineText()
		self["Event"] = Event()
		self["ExtEvent"] = ExtEvent()
		self["JumpInterval"] = Label(_("jump: ") + " " + config.misc.graph_mepg_prev_time_jump.value + " " + _("h"))
		self.JumpInterval = int(config.misc.graph_mepg_prev_time_jump.value)
		self.timeframe = int(config.misc.graph_mepg_prev_time_period.value)
		self.time_lines = [ ]
		for x in (0,1,2,3,4,5):
			pm = Pixmap()
			self.time_lines.append(pm)
			self["timeline%d"%(x)] = pm
		self["timeline_now"] = Pixmap()
		self.services = services
		self.zapFunc = zapFunc
		self.red_long = False
		self["list"] = EPGList(selChangedCB = self.onSelectionChanged, timer = self.session.nav.RecordTimer, time_epoch = int(config.misc.graph_mepg_prev_time_period.value), skinname = self.skinName)

		self["WizardActions"] = HelpableActionMap(self, "WizardActions", 
			{
				"back": (self.closeScreen, _("Exit plugin")),
				"ok": (self.keyOK, _(config.misc.graph_mepg_button_ok.getText())),
				"left": (self.leftPressed,_("Previous entry")),
				"right": (self.rightPressed,_("Next entry")),
				"up": (self.ChannelUp,_("Next Channel")),
				"down": (self.ChannelDown,_("Previous Channel")),
			}, -1)
		
		if config.misc.graph_mepg_button_yb.value:
			self["key_yellow"] = Button(_("Previous Bouquet") + "\n" + _("Timer Editor"))
			self["key_blue"] = Button(_("Next Bouquet") + "\n" + _("EPG search"))
			self["ColorLongActions"] = HelpableActionMap(self, "ColorLongActions", 
				{
					"red": (self.zapExit, _("Zap")),
					"green": (self.timerAdd, _("Add timer") + " (" + _("Timer Editor") + ")"),
					"yellow": (self.prevBouquet, _("Previous Bouquet")),
					"blue": (self.nextBouquet, _("Next Bouquet")),
					"red_long": (self.zapTo, _("Preview")),
					"green_long": (self.timerEnableDisable, _("Toggle timer state")),
					"yellow_long": (self.Timer_List, _("Timer Editor")),
					"blue_long": (self.EpgSearch, _("EPG search")),
				}, -1)
			self["EPGSelectActions"] = HelpableActionMap(self, "EPGSelectActions",
				{
					"instantToggleTimerState": (self.instantToggleTimerState, _("Toggle timer state")),
					"instantTimer": (self.addInstantTimer,_("Add instant Timer")),
					"input_date_time": (self.goMenu, _("Settings")),
					"nextBouquet": (self.ChannelsUp,_("Page up")),
					"prevBouquet": (self.ChannelsDown,_("Page down")),
					"nextService": (self.jumpDayforward,_("Jump Day forward")),
					"prevService": (self.jumpDayback,_("Jump Day backward")),
				}, -1)
		else:
			self["key_yellow"] = Button(_("Page up") + "\n" + _("Timer Editor"))
			self["key_blue"] = Button(_("Page down") + "\n" + _("EPG search"))
			self["ColorLongActions"] = HelpableActionMap(self, "ColorLongActions", 
				{
					"red": (self.zapExit, _("Zap")),
					"green": (self.timerAdd, _("Add timer") + " (" + _("Timer Editor") + ")"),
					"yellow": (self.ChannelsUp, _("Page up")),
					"blue": (self.ChannelsDown, _("Page down")),
					"red_long": (self.zapTo, _("Preview")),
					"green_long": (self.timerEnableDisable, _("Toggle timer state")),
					"yellow_long": (self.Timer_List, _("Timer Editor")),
					"blue_long": (self.EpgSearch, _("EPG search")),
				}, -1)
			self["EPGSelectActions"] = HelpableActionMap(self, "EPGSelectActions",
				{
					"instantToggleTimerState": (self.instantToggleTimerState, _("Toggle timer state")),
					"instantTimer": (self.addInstantTimer,_("Add instant Timer")),
					"input_date_time": (self.goMenu, _("Settings")),
					"nextBouquet": (self.nextBouquet,_("Next Bouquet")),
					"prevBouquet": (self.prevBouquet,_("Previous Bouquet")),
					"nextService": (self.jumpDayforward,_("Jump Day forward")),
					"prevService": (self.jumpDayback,_("Jump Day backward")),
				}, -1)
			
		self["TeletextActions"] = HelpableActionMap(self, "InfobarTeletextActions",
			{
				"startTeletext": (self.infoKeyPressed,_("Switch between views")),
			}, -1)

		self["input_actions"] = HelpableActionMap(self,"InputActions",
			{
				"1": (self.key1,_("Show less entries")),
				"2": (self.key2,_("Show normal entries")),
				"3": (self.key3,_("Show more entries")),
				"4": (self.key4,_("Jump prefered hours backward")),
				"5": (self.key5,_("Set prefered hours to jump")),
				"6": (self.key6,_("Jump prefered hours forward")),
				"7": (self.key7,_("Jump page backward")),
				"8": (self.key8,_("Jump to Prime Time")),
				"9": (self.key9,_("Jump page forward")),
				"0": (self.key0,_("Jump to now")),
			},-1)
			
		self["InfobarActions"] = HelpableActionMap(self, "InfobarActions", 
			{
				"showSubtitle": (self.clock2bar,_("Toggle clock / bars view for timer")),
				"showMovies": (self.addAutotimer, _("Add Autotimer")),
			}, -1)
		self["TimerEditActions"] = HelpableActionMap(self, "TimerEditActions", 
			{
				"eventview": (self.keyEPG,_(config.misc.graph_mepg_button_epg.getText())),
				"log": (self.tmdbSearch, _("TMDb Infos") + "..."),
			}, -1)
		
		self.updateTimelineTimer = eTimer()
		self.updateTimelineTimer.callback.append(self.moveTimeLines)
		self.updateTimelineTimer.start(60 * 1000)
		self.onLayoutFinish.append(self.onCreate)
		self.onLayoutFinish.append(self.onLayout)

	def onLayout(self):
		bouquet_name = self.bouquetName
		if self.bouquetName is None:
			bouquet_name = ""
		self.setTitle(bouquet_name)
		self["JumpInterval"].setText(_("jump: ") + " " + str(self.JumpInterval) + " " + _("h"))

	def QuickZoom(self):
		if config.misc.graph_mepg_zoom.value == True:
			self["zoom"].hide()
			self["zoomBack"].hide()
			self["zoomShadow"].hide()
			
			l = self["list"]
			cur = l.getCurrent()
			if cur[0] is not None:
				list_x,list_y = l.position
				list_w = l.instance.size().width()
				list_h = l.instance.size().height()
				dx,dw = l.getEntry()
				event_rect = l.getEventRect()
				dh = event_rect.height()
				list_rows = int(list_h/dh)
				list_length = int(l.instance.getCurrentIndex() % list_rows)
				dy = list_length * dh
				
				if dw <= list_w/6:
					event = cur[0]
					eventtime = cur[0].getBeginTime()
					t = localtime(eventtime)
					eventTime = "%02d:%02d"%(t[3],t[4])
					eventName = cur[0].getEventName()
					eventDesc = ""
					eventDesc = cur[0].getShortDescription()
					pos_x = 0
					pos_y = 0
					
					font, size = skin.parameters.get("GMultiEPGZoomFont", ('Regular',20))
					df = int(config.misc.graph_mepg_fontsize.value)
					self["zoom"].instance.setFont(gFont(font, size + df))
					self["zoom"].instance.setNoWrap(1)
					self["zoom"].setText(" " + eventTime + "\n " + eventName + "\n " + eventDesc)
					labelsize = self["zoom"].instance.calculateSize()
					label_size_w = labelsize.width() + 7
					label_size_h = labelsize.height() + 13
					
					if dx >= list_w/2:
						pos_x =list_x+dx+dw-label_size_w-2
					else:
						pos_x =list_x+dx

					if dy >= list_h/2:
						pos_y =list_y+dy-label_size_h-2
					else:
						pos_y =list_y+dy+dh
	
					self["zoomBack"].move(pos_x,pos_y)
					self["zoomShadow"].move(pos_x+6,pos_y+6)
					self["zoom"].move(pos_x+2,pos_y+2)
					self["zoom"].resize(label_size_w,label_size_h)
					self["zoomBack"].resize(label_size_w+4,label_size_h+4)
					self["zoomShadow"].resize(label_size_w+4,label_size_h+4)
					self["zoom"].show()
					self["zoomBack"].show()
					self["zoomShadow"].show()

	def goMenu(self):
		self.session.openWithCallback(self.goRestart, GraphMultiEPGConfigScreen)

	def goRestart(self):
		self["list"].setEpoch(self.timeframe)
		self.moveTimeLines()		
		
	def Timer_List(self):
		from Screens.TimerEdit import TimerEditList
		self.session.open(TimerEditList)

	def EpgSearch(self):
		try:
			from Plugins.Extensions.EPGSearch.EPGSearch import EPGSearch
			plugin_installed = True
		except ImportError:
			plugin_installed = False
			plugin_name = "EPGSearch"
			self.session.open(MessageBox, _("Sorry, but the plugin %s is not installed at your Vu+ STB!\nPlease install it to use this function") % plugin_name, type = MessageBox.TYPE_ERROR, timeout = 10)
		if plugin_installed:
			cur = self["list"].getCurrent()
			event = cur[0]
			if event is not None:
				self.session.open(EPGSearch, event.getEventName(), False)

	def tmdbSearch(self):
		plugin_installed = False
		try:
			from Plugins.Extensions.tmdb.plugin import tmdbScreen
			plugin_installed = True
		except ImportError:
			try:
				from Plugins.Extensions.tmdb.tmdb import tmdbScreen
				plugin_installed = True
			except ImportError:
				self.imdbSearch()
		if plugin_installed:
			cur = self["list"].getCurrent()
			event = cur[0]
			if event is not None:
				self.session.open(tmdbScreen, event.getEventName(), 2)
				
	def imdbSearch(self):
		try:
			from Plugins.Extensions.IMDb.plugin import IMDB
			plugin_installed = True
		except ImportError:
			plugin_installed = False
			plugin_name = "TMDb/IMDb"
			self.session.open(MessageBox, _("Sorry, but the plugin %s is not installed at your Vu+ STB!\nPlease install it to use this function") % plugin_name, type = MessageBox.TYPE_ERROR, timeout = 10)
		if plugin_installed:
			cur = self["list"].getCurrent()
			event = cur[0]
			if event is not None:
				self.session.open(IMDB, event.getEventName(), 2)			
				
	def addAutotimer(self):
		try:
			from Plugins.Extensions.AutoTimer.AutoTimerEditor import addAutotimerFromEvent
			plugin_installed = True
		except ImportError:
			plugin_installed = False
			plugin_name = "AutoTimer"
			self.session.open(MessageBox, _("Sorry, but the plugin %s is not installed at your Vu+ STB!\nPlease install it to use this function") % plugin_name, type = MessageBox.TYPE_ERROR, timeout = 10)
		if plugin_installed:
			cur = self["list"].getCurrent()
			evt = cur[0]
			sref = cur[1]
			if not evt:
				return
			self.session.openWithCallback(self.addAutotimerCB, ChoiceBox, title=_('Check Autotimer?'), list=[(_('Yes'), True),(_('No'), False)])	
			addAutotimerFromEvent(self.session, evt = evt, service = sref)
			
	def addAutotimerCB(self, answer):
		answer = answer and answer[1]
		if answer:
			from Plugins.Extensions.AutoTimer.plugin import main as checkAutoTimer
			checkAutoTimer(self.session)

	def clock2bar(self):
		if config.misc.graph_mepg_bars.value:
			config.misc.graph_mepg_bars.value = False
		else:
			config.misc.graph_mepg_bars.value = True
		self.moveTimeLines()
	
	def key1(self):
		if self.timeframe > 60:
			self.timeframe = self.timeframe - 60
		else:
			self.timeframe = 60
		self["list"].setEpoch(self.timeframe)
		self.moveTimeLines()

	def key2(self):
		self["list"].setEpoch(120)
		self.moveTimeLines()

	def key3(self):
		if self.timeframe < 360:
			self.timeframe = self.timeframe + 60
		else:
			self.timeframe = 360
		self["list"].setEpoch(self.timeframe)
		self.moveTimeLines()
	
	def key4(self):
		l = self["list"]
		jump = l.getTimeBase() - self.JumpInterval*3600
		if self.ask_time > jump:
			now = time()
			tmp = now % 900
			self.ask_time = now - tmp
			jump = self.ask_time
		l.resetOffset()
		l.fillMultiEPG(self.services, jump)
		self.moveTimeLines(True)	

	def key5(self):
		if self.JumpInterval < (24):
			self.JumpInterval += (1)
		else:
			self.JumpInterval = (1)
		self.onLayout()
	
	def key6(self):
		l = self["list"]
		jump = l.getTimeBase() + self.JumpInterval*3600
		l.resetOffset()
		l.fillMultiEPG(self.services, jump)
		self.moveTimeLines(True)
	
	def key7(self):
		l = self["list"]
		jump = l.getTimeBase() - (self.timeframe*60)
		if self.ask_time > jump:
			now = time()
			tmp = now % 900
			self.ask_time = now - tmp
			jump = self.ask_time
		l.resetOffset()
		l.fillMultiEPG(self.services, jump)
		self.moveTimeLines(True)
		
	def key8(self):
		primetime = (config.misc.graph_mepg_prev_time.value[0] * 60 + config.misc.graph_mepg_prev_time.value[1])*60
		loctime = localtime()
		acttime = (loctime[3] * 60 + loctime[4])*60
		now = time()
		jump2prime = int(now - acttime) + primetime
		l = self["list"]
		l.resetOffset()
		l.fillMultiEPG(self.services, jump2prime)
		self.moveTimeLines(True)
	
	def key9(self):
		l = self["list"]
		jump = l.getTimeBase() + (self.timeframe*60)
		l.resetOffset()
		l.fillMultiEPG(self.services, jump)
		self.moveTimeLines(True)
		
	def key0(self):
		now = time()
		tmp = now % 900
		jump2now = now - tmp
		l = self["list"]
		l.resetOffset()
		l.fillMultiEPG(self.services, jump2now)
		self.moveTimeLines(True)

	def leftPressed(self):
		self.prevEvent()

	def rightPressed(self):
		self.nextEvent()

	def nextEvent(self, visible=True):
		self["list"].selEntry(0)
		ret = self["list"].selEntry(+1, visible)
		if ret:
			self.moveTimeLines(True)

	def prevEvent(self, visible=True):
		self["list"].selEntry(0)
		ret = self["list"].selEntry(-1, visible)
		if ret:
			self.moveTimeLines(True)

	def ChannelsDown(self):
		self["list"].selEntry(0)
		self["list"].instance.moveSelection(self["list"].instance.pageDown)
		self.zapAuto()
	
	def ChannelsUp(self):
		self["list"].selEntry(0)
		self["list"].instance.moveSelection(self["list"].instance.pageUp)
		self.zapAuto()
		
	def ChannelDown(self):
		self["list"].selEntry(0)
		self["list"].instance.moveSelection(self["list"].instance.moveDown)
		self.zapAuto()
		
	def ChannelUp(self):
		self["list"].selEntry(0)
		self["list"].instance.moveSelection(self["list"].instance.moveUp)
		self.zapAuto()
		
	def nextBouquet(self):
		l = self["list"]
		jump = l.getTimeBase()
		if self.bouquetChangeCB:
			self.bouquetName = self.bouquetChangeCB(1, self)
			self.onLayout()
		l.resetOffset()
		l.fillMultiEPG(self.services, jump)
		self.zapAuto()
		
	def prevBouquet(self):
		l = self["list"]
		jump = l.getTimeBase()
		if self.bouquetChangeCB:
			self.bouquetName = self.bouquetChangeCB(-1, self)
			self.onLayout()
		l.resetOffset()
		l.fillMultiEPG(self.services, jump)
		self.zapAuto()
		
	def enterDateTime(self):
		self.session.openWithCallback(self.onDateTimeInputClosed, TimeDateInput, config.misc.graph_mepg_prev_time )

	def onDateTimeInputClosed(self, ret):
		if len(ret) > 1:
			if ret[0]:
				self.ask_time=ret[1]
				l = self["list"]
				l.resetOffset()
				l.fillMultiEPG(self.services, ret[1])
				self.moveTimeLines(True)

	def jumpDayforward(self):
		l = self["list"]
		jump24 = l.getTimeBase() + 86400
		l.resetOffset()
		l.fillMultiEPG(self.services, jump24)
		self.moveTimeLines(True)

	def jumpDayback(self):
		l = self["list"]
		jump24 = l.getTimeBase() - 86400
		if self.ask_time > jump24:
			now = time()
			tmp = now % 900
			self.ask_time = now - tmp
			jump24 = self.ask_time
		l.resetOffset()
		l.fillMultiEPG(self.services, jump24)
		self.moveTimeLines(True)
		
	def closeScreen(self):
		self.close(self.closeRecursive)

	def infoKeyPressed(self):
		if config.misc.graph_mepg_skin.value == "SKIN0":
			config.misc.graph_mepg_skin.value = "SKIN1"
			self.goRestart()
		elif config.misc.graph_mepg_skin.value == "SKIN1":
			config.misc.graph_mepg_skin.value = "SKIN2"
			self.goRestart()
		elif config.misc.graph_mepg_skin.value == "SKIN2":
			config.misc.graph_mepg_skin.value = "SKIN3"
			self.goRestart()
		elif config.misc.graph_mepg_skin.value == "SKIN3":
			config.misc.graph_mepg_skin.value = "SKIN0"
			self.goRestart()
		
	def goRestart(self):
		if config.usage.multiepg_ask_bouquet.value:
			config.misc.graph_mepg_prev_time_jump.save()
			self.close(self.closeRecursive)
		else:
			config.misc.graph_mepg_prev_time_jump.save()
			config.misc.graph_mepg_skin.save()
			self.close(None)

	def openSimilarList(self, eventid, refstr):
		self.session.open(EPGSelection, refstr, None, eventid)

	def setServices(self, services):
		self.services = services
		self.onCreate()

	def onCreate(self):
		self["list"].fillMultiEPG(self.services, self.ask_time)
		self["list"].moveToService(self.session.nav.getCurrentlyPlayingServiceReference())
		self.moveTimeLines()

	def updateList(self):
		self["list"].reload()

	def eventViewCallback(self, setEvent, setService, val):
		l = self["list"]
		old = l.getCurrent()
		if val == -1:
			self.prevEvent(False)
		elif val == +1:
			self.nextEvent(False)
		cur = l.getCurrent()
		if cur[0] is None and cur[1].ref != old[1].ref:
			self.eventViewCallback(setEvent, setService, val)
		else:
			setService(cur[1])
			setEvent(cur[0])

	def zapTo(self):
		self.red_long = True
		if self.zapFunc and self.key_red_choice == self.ZAP:
			self.closeRecursive = True
			ref = self["list"].getCurrent()[1]
			if ref:
				self.zapFunc(ref.ref)

	def zapExit(self):
		if self.red_long:
			self.red_long = False
		else:
			if self.zapFunc and self.key_red_choice == self.ZAP:
				self.closeRecursive = True
				ref = self["list"].getCurrent()[1]
				if ref:
					self.zapFunc(ref.ref)
				self.close(False)

	def zapAuto(self):				
		if config.misc.graph_mepg_autozap.value and self.doAutoZap:
			self.zapTo()

	def eventSelected(self):
		cur = self["list"].getCurrent()
		event = cur[0]
		service = cur[1]
		if event is not None:
			self.session.open(EventViewSimple, event, service, self.eventViewCallback, self.openSimilarList)

	def singleServiceEPGZap(self, ref, check_bouquet = None):
		if self.zapFunc and self.key_red_choice == self.ZAP:
			self.closeRecursive = True
			if ref:
				self.zapFunc(ref)
			self.close(False)

	def keyOK(self):
		if config.misc.graph_mepg_button_ok.value == "1":
			self.openSingleServiceEPG()
		elif config.misc.graph_mepg_button_ok.value == "2":
			self.eventSelected()
		elif config.misc.graph_mepg_button_ok.value == "3":
			self.zapExit()
		elif config.misc.graph_mepg_button_ok.value == "4":
			self.zapTo()
			
	def keyEPG(self):
		if config.misc.graph_mepg_button_epg.value == "1":
			self.openSingleServiceEPG()
		elif config.misc.graph_mepg_button_epg.value == "2":
			self.eventSelected()
		elif config.misc.graph_mepg_button_epg.value == "3":
			self.zapExit()
		elif config.misc.graph_mepg_button_ok.value == "4":
			self.zapTo()
			
	def openSingleServiceEPG(self):
		cur = self["list"].getCurrent()
		if len(cur) >= 2:
			service = cur[1]
			if service is not None:
				self.session.openWithCallback(self.SingleServiceEPGClosed, EPGSelection, service.ref, zapFunc = self.singleServiceEPGZap)

	def SingleServiceEPGClosed(self, ret = None):
		pass

	def removeTimer(self, timer, instantTimer = False):
		timer.afterEvent = AFTEREVENT.NONE
		self.session.nav.RecordTimer.removeEntry(timer)
		self["key_green"].setText(_("Add timer"))
		self.key_green_choice = self.ADD_TIMER
		if instantTimer:
			self.updateList()

	def addInstantTimer(self):
		self.timerAdd(True)

	def timerAdd(self, instantTimer = False):
		cur = self["list"].getCurrent()
		event = cur[0]
		serviceref = cur[1]
		if event is None:
			return
		eventid = event.getEventId()
		refstr = serviceref.ref.toString()
		
		if self.key_green_choice == self.ENABLE_TIMER:
			for timer in self.session.nav.RecordTimer.processed_timers:
				if timer.eit == eventid and timer.service_ref.ref.toString() == refstr and timer.disabled == True:
					if instantTimer:
						self.removeTimer(timer, True)
					else:
						cb_func = lambda ret : not ret or self.removeTimer(timer)
						self.session.openWithCallback(cb_func, MessageBox, _("Do you really want to delete %s?") % event.getEventName())
					break
		else:
			for timer in self.session.nav.RecordTimer.timer_list:
				if timer.eit == eventid and timer.service_ref.ref.toString() == refstr:
					if instantTimer:
						self.removeTimer(timer, True)
						break
					else:
						cb_func = lambda ret : not ret or self.removeTimer(timer)
						self.session.openWithCallback(cb_func, MessageBox, _("Do you really want to delete %s?") % event.getEventName())
						break
			else:
				newEntry = RecordTimerEntry(serviceref, checkOldTimers = True, allow_duplicate = config.usage.timer_allow_duplicate.value, *parseEvent(event))
				if instantTimer:
					if config.usage.timer_ask_path.value and config.movielist.videodirs.value and len(config.movielist.videodirs.value) > 1:
						choicelist = []
						for x in config.movielist.videodirs.value:
							choicelist.append((x,x))
						idx = 0
						pref = (preferredTimerPath(), preferredTimerPath())
						if pref in choicelist:
							idx = choicelist.index(pref)
						self.session.openWithCallback(self.askTimerPath, ChoiceBox, title=_("Timer record location"), list = choicelist, selection = idx)
					else:
						self.session.nav.RecordTimer.saveTimer()
						self.finishedAdd((True, newEntry), True)
				else:
					if AutoTimerHook.AUTOTIMER_OK and config.usage.ask_for_timer_typ.value:
						self.session.openWithCallback(self.askTimerType, ChoiceBox, title=_("Select timer type..."), list = AutoTimerHook.getChoiceList())
					else:
						self.session.openWithCallback(self.finishedAdd, TimerEntry, newEntry)

	def askTimerPath(self, ret = None):
		if ret and ret[0] and os.path.exists(ret[0]):
			cur = self["list"].getCurrent()
			event = cur[0]
			serviceref = cur[1]
			if event is None:
				return
			newEntry = RecordTimerEntry(serviceref, checkOldTimers = True, dirname = ret[0], allow_duplicate = config.usage.timer_allow_duplicate.value, *parseEvent(event))
			self.session.nav.RecordTimer.saveTimer()
			self.finishedAdd((True, newEntry), True)

	def askTimerType(self, ret = None):
		if ret:
			cur = self["list"].getCurrent()
			event = cur[0]
			serviceref = cur[1]
			if event is None:
				return
			if ret[1] == "default":
				newEntry = RecordTimerEntry(serviceref, checkOldTimers = True, dirname = preferredTimerPath(), allow_duplicate = config.usage.timer_allow_duplicate.value, *parseEvent(event))
				self.session.openWithCallback(self.finishedAdd, TimerEntry, newEntry)
			elif ret[1] == "autotimer":
				AutoTimerHook.addAutotimerFromEvent(self.session,evt = event, service = serviceref)
			elif ret[1] == "show_autotimer":
				AutoTimerHook.AutoTimerOverView(self.session)
			elif ret[1] == "show_timerlist":
				self.session.open(TimerEditList)

	def finishedAdd(self, answer, instantTimer = False):
		vtilog("finished add")
		if answer[0]:
			entry = answer[1]
			simulTimerList = self.session.nav.RecordTimer.record(entry)
			if simulTimerList is not None:
				for x in simulTimerList:
					if x.setAutoincreaseEnd(entry):
						self.session.nav.RecordTimer.timeChanged(x)
				simulTimerList = self.session.nav.RecordTimer.record(entry)
				if simulTimerList is not None:
					self.session.openWithCallback(self.finishSanityCorrection, TimerSanityConflict, simulTimerList)
			self["key_green"].setText(_("Remove timer") + "\n" + _("Disable timer"))
			self.key_green_choice = self.REMOVE_TIMER
			if instantTimer:
				self.updateList()
		else:
			self["key_green"].setText(_("Add timer"))
			self.key_green_choice = self.ADD_TIMER
			vtilog("Timeredit aborted")
	
	def finishSanityCorrection(self, answer):
		self.finishedAdd(answer)

	def instantToggleTimerState(self):
		self.timerEnableDisable(True)

	def timerEnableDisable(self, instant_toggle = False):
		cur = self["list"].getCurrent()
		event = cur[0]
		serviceref = cur[1]
		if event is None:
			return
		eventid = event.getEventId()
		refstr = serviceref.ref.toString()
		for timer in self.session.nav.RecordTimer.timer_list:
			if timer.eit == eventid and timer.service_ref.ref.toString() == refstr:
				if instant_toggle:
					self.toggleTimerState(timer, True)
				else:
					cb_func = lambda ret : not ret or self.toggleTimerState(timer, True)
					self.session.openWithCallback(cb_func, MessageBox, _("Do you really want to disable timer:") + "\n" + event.getEventName() + " ?")
				return
		for timer in self.session.nav.RecordTimer.processed_timers:
			if timer.eit == eventid and timer.service_ref.ref.toString() == refstr and timer.disabled == True:
				if instant_toggle:
					self.toggleTimerState(timer, False)
				else:
					cb_func = lambda ret : not ret or self.toggleTimerState(timer, False)
					self.session.openWithCallback(cb_func, MessageBox, _("Do you really want to enable timer:") + "\n" + event.getEventName() + " ?")
				return

	def toggleTimerState(self, timer, disableTimer):
		if disableTimer:
			timer.disable()
			self.session.nav.RecordTimer.timeChanged(timer)
			self["key_green"].setText(_("Remove") + "\n" + _("Enable timer"))
			self.key_green_choice = self.ENABLE_TIMER
		else:
			timer.enable()
			timersanitycheck = TimerSanityCheck(self.session.nav.RecordTimer.timer_list, timer)
			if not timersanitycheck.check():
				timer.disable()
				self["key_green"].setText("")
				self.key_green_choice = self.EMPTY
				simulTimerList = timersanitycheck.getSimulTimerList()
				if simulTimerList is not None:
					self.session.openWithCallback(self.finishedEdit, TimerSanityConflict, simulTimerList)
			else:
				if timersanitycheck.doubleCheck():
					timer.disable()
					self["key_green"].setText(_("Remove") + "\n" + _("Enable timer"))
					self.key_green_choice = self.REMOVE_TIMER
			self.session.nav.RecordTimer.timeChanged(timer)
		self.updateList()

	def finishedEdit(self, answer):
		if answer[0]:
			entry = answer[1]
			timersanitycheck = TimerSanityCheck(self.session.nav.RecordTimer.timer_list, entry)
			success = False
			if not timersanitycheck.check():
				simulTimerList = timersanitycheck.getSimulTimerList()
				if simulTimerList is not None:
					for x in simulTimerList:
						if x.setAutoincreaseEnd(entry):
							self.session.nav.RecordTimer.timeChanged(x)
					if not timersanitycheck.check():
						simulTimerList = timersanitycheck.getSimulTimerList()
						if simulTimerList is not None:
							self.session.openWithCallback(self.finishedEdit, TimerSanityConflict, timersanitycheck.getSimulTimerList())
					else:
						success = True
			else:
				success = True
			if success:
				self.session.nav.RecordTimer.timeChanged(entry)
			self.updateList()

	def onSelectionChanged(self):
		self.QuickZoom()
		
		cur = self["list"].getCurrent()
		if cur is None:
			if self.key_green_choice != self.EMPTY:
				self["key_green"].setText("")
				self.key_green_choice = self.EMPTY
			if self.key_red_choice != self.EMPTY:
				self["key_red"].setText("")
				self.key_red_choice = self.EMPTY
			return
		
		event = cur[0]
		self["Event"].newEvent(event)
		self["ExtEvent"].newEvent(cur[0])
		if cur[1] is None or cur[1].getServiceName() == "":
			if self.key_green_choice != self.EMPTY:
				self["key_green"].setText("")
				self.key_green_choice = self.EMPTY
			if self.key_red_choice != self.EMPTY:
				self["key_red"].setText("")
				self.key_red_choice = self.EMPTY
			return
		elif self.key_red_choice != self.ZAP:
				self["key_red"].setText(_("Zap") + "\n" + _("Preview"))
				self.key_red_choice = self.ZAP
			
		if not event:
			if self.key_green_choice != self.EMPTY:
				self["key_green"].setText("")
				self.key_green_choice = self.EMPTY
			return
		
		serviceref = cur[1]
		self["ExtEvent"].newService(cur[1])
		eventid = event.getEventId()
		refstr = serviceref.ref.toString()
		isRecordEvent = False
		isDisabled = False
		for timer in self.session.nav.RecordTimer.timer_list:
			if timer.eit == eventid and timer.service_ref.ref.toString() == refstr:
				isRecordEvent = True
				break
		if not isRecordEvent:
			for timer in self.session.nav.RecordTimer.processed_timers:
				if timer.eit == eventid and timer.service_ref.ref.toString() == refstr and timer.disabled == True:
					isDisabled = True
					break
		if isDisabled:
			self["key_green"].setText(_("Remove timer") + "\n" + _("Enable timer"))
			self.key_green_choice = self.ENABLE_TIMER
		elif  isRecordEvent:
			self["key_green"].setText(_("Remove timer") + "\n" + _("Disable timer"))
			self.key_green_choice = self.REMOVE_TIMER
		elif not isRecordEvent and self.key_green_choice != self.ADD_TIMER:
			self["key_green"].setText(_("Add timer"))
			self.key_green_choice = self.ADD_TIMER
	
	def moveTimeLines(self, force=False):
		self.updateTimelineTimer.start((60 - (int(time()) % 60)) * 1000)
		l = self["list"]
		event_rect = l.getEventRect()
		time_epoch = l.getTimeEpoch()
		time_base = l.getTimeBase()
		if event_rect is None or time_epoch is None or time_base is None:
			return
		time_steps = time_epoch > 180 and 60 or 30
		
		num_lines = time_epoch/time_steps
		incWidth=event_rect.width()/num_lines
		pos=event_rect.left()
		timeline_entries = [ ]
		x = 0
		changecount = 0
		for line in self.time_lines:
			old_pos = line.position
			new_pos = (x == num_lines and event_rect.left()+event_rect.width()or pos, old_pos[1])
			if not x or x >= num_lines:
				line.visible = False
			else:
				if old_pos != new_pos:
					line.setPosition(new_pos[0], new_pos[1])
					changecount += 1
				line.visible = True
			if not x or line.visible:
				timeline_entries.append((time_base + x * time_steps * 60, new_pos[0]))
			x += 1
			pos += incWidth

		if changecount or force:
			self["timeline_text"].setEntries(timeline_entries)

		now=time()
		timeline_now = self["timeline_now"]
		if now >= time_base and now < (time_base + time_epoch * 60):
			xpos = int((((now - time_base) * event_rect.width()) / (time_epoch * 60))-(timeline_now.instance.size().width()/2))
			old_pos = timeline_now.position
			new_pos = (xpos+event_rect.left(), old_pos[1])
			if old_pos != new_pos:
				timeline_now.setPosition(new_pos[0], new_pos[1])
			timeline_now.visible = True
		else:
			timeline_now.visible = False
		l.l.invalidate()
